@@ -65,11 +65,13 @@
     },
     makeCheckDescriptor: function(descriptor) {
       // Assert descriptor is non-null, then that its nextInChain is null.
-      // For descriptors that aren't the first in the chain (e.g ShaderModuleSPIRVDescriptor),
-      // there is no nextInChain pointer but a ChainedStruct object named chain.
-      // So we need to check if chain.nextInChain is null. As long as nextInChain and chain are both the 
-      // first member in the struct, descriptor.nextInChain and descriptor.chain.nextInChain should have the same offset (0)
-      // to the descriptor pointer and we can check it to be null.
+      // For descriptors that aren't the first in the chain (e.g
+      // ShaderModuleSPIRVDescriptor), there is no .nextInChain pointer, but
+      // instead a ChainedStruct object: .chain. So we need to check if
+      // .chain.nextInChain is null. As long as nextInChain and chain are always
+      // the first member in the struct, descriptor.nextInChain and
+      // descriptor.chain.nextInChain should have the same offset (0) to the
+      // descriptor pointer and we can check it to be null.
       var OffsetOfNextInChainMember = 0;
       return this.makeCheck(descriptor) + this.makeCheck(makeGetValue(descriptor, OffsetOfNextInChainMember, '*') + ' === 0');
     },
@@ -79,7 +81,7 @@
         Fifo: 2,
     },
     SType: {
-        SurfaceDescriptorFromHTMLCanvasId: 4,
+        SurfaceDescriptorFromCanvasHTMLSelector: 4,
         ShaderModuleSPIRVDescriptor: 5,
         ShaderModuleWGSLDescriptor: 6,
     },
@@ -94,11 +96,12 @@ var LibraryWebGPU = {
 
       function makeManager() {
         return {
-          objects: [undefined],
+          objects: {},
+          nextId: 1,
           create: function(object, wrapper /* = {} */) {
             wrapper = wrapper || {};
 
-            var id = this.objects.length;
+            var id = this.nextId++;
             {{{ gpu.makeCheck("typeof this.objects[id] === 'undefined'") }}}
             wrapper.refcount = 1;
             wrapper.object = object;
@@ -145,6 +148,7 @@ var LibraryWebGPU = {
       {{{ gpu.makeInitManager('Sampler') }}}
       {{{ gpu.makeInitManager('Texture') }}}
       {{{ gpu.makeInitManager('TextureView') }}}
+      {{{ gpu.makeInitManager('QuerySet') }}}
 
       {{{ gpu.makeInitManager('BindGroupLayout') }}}
       {{{ gpu.makeInitManager('PipelineLayout') }}}
@@ -156,21 +160,6 @@ var LibraryWebGPU = {
       {{{ gpu.makeInitManager('RenderBundle') }}}
     },
 
-    trackMapWrite: function(obj, mapped) {
-      var data = _malloc(mapped.byteLength);
-      HEAPU8.fill(0, data, mapped.byteLength);
-      obj.mapWriteSrc = data;
-      obj.mapWriteDst = mapped;
-    },
-    trackUnmap: function(obj) {
-      if (obj.mapWriteSrc) {
-        new Uint8Array(obj.mapWriteDst).set(HEAPU8.subarray(obj.mapWriteSrc, obj.mapWriteSrc + obj.mapWriteDst.byteLength));
-        _free(obj.mapWriteSrc);
-      }
-      obj.mapWriteSrc = undefined;
-      obj.mapWriteDst = undefined;
-    },
-
     makeColor: function(ptr) {
       return {
         "r": {{{ makeGetValue('ptr', 0, 'float') }}},
@@ -202,20 +191,26 @@ var LibraryWebGPU = {
         "texture": this.mgrTexture.get(
           {{{ makeGetValue('ptr', C_STRUCTS.WGPUTextureCopyView.texture, '*') }}}),
         "mipLevel": {{{ gpu.makeGetU32('ptr', C_STRUCTS.WGPUTextureCopyView.mipLevel, '*') }}},
-        "arrayLayer": {{{ gpu.makeGetU32('ptr', C_STRUCTS.WGPUTextureCopyView.arrayLayer, '*') }}},
         "origin": WebGPU.makeOrigin3D(ptr + {{{ C_STRUCTS.WGPUTextureCopyView.origin }}}),
       };
     },
 
+    makeTextureDataLayout: function(ptr) {
+      {{{ gpu.makeCheckDescriptor('ptr') }}}
+      return {
+        "offset": {{{ gpu.makeGetU64('ptr', C_STRUCTS.WGPUTextureDataLayout.offset) }}},
+        "bytesPerRow": {{{ gpu.makeGetU32('ptr', C_STRUCTS.WGPUTextureDataLayout.bytesPerRow) }}},
+        "rowsPerImage": {{{ gpu.makeGetU32('ptr', C_STRUCTS.WGPUTextureDataLayout.rowsPerImage) }}},
+      };
+    },
+
     makeBufferCopyView: function(ptr) {
       {{{ gpu.makeCheckDescriptor('ptr') }}}
-      return {
-        "buffer": this.mgrBuffer.get(
-          {{{ makeGetValue('ptr', C_STRUCTS.WGPUBufferCopyView.buffer, '*') }}}),
-        "offset": {{{ gpu.makeGetU64('ptr', C_STRUCTS.WGPUBufferCopyView.offset) }}},
-        "bytesPerRow": {{{ gpu.makeGetU32('ptr', C_STRUCTS.WGPUBufferCopyView.bytesPerRow) }}},
-        "rowsPerImage": {{{ gpu.makeGetU32('ptr', C_STRUCTS.WGPUBufferCopyView.rowsPerImage) }}},
-      };
+      var layoutPtr = ptr + {{{ C_STRUCTS.WGPUBufferCopyView.layout }}};
+      var bufferCopyView = this.makeTextureDataLayout(layoutPtr);
+      bufferCopyView["buffer"] = this.mgrBuffer.get(
+          {{{ makeGetValue('ptr', C_STRUCTS.WGPUBufferCopyView.buffer, '*') }}});
+      return bufferCopyView;
     },
 
     makeProgrammableStageDescriptor: function(ptr) {
@@ -248,7 +243,6 @@ var LibraryWebGPU = {
       'sampler',
       'comparison-sampler',
       'sampled-texture',
-      undefined, // WGPUBindingType_StorageTexture (deprecated)
       'readonly-storage-texture',
       'writeonly-storage-texture',
     ],
@@ -334,6 +328,13 @@ var LibraryWebGPU = {
       'clear',
       'load',
     ],
+    PipelineStatisticName: [
+      'vertex-shader-invocations',
+      'clipper-invocations',
+      'clipper-primitives-out',
+      'fragment-shader-invocations',
+      'compute-shader-invocations',
+    ],
     PrimitiveTopology: [
       'point-list',
       'line-list',
@@ -341,6 +342,11 @@ var LibraryWebGPU = {
       'triangle-list',
       'triangle-strip',
     ],
+    QueryType: [
+      'occlusion',
+      'pipeline-statistics',
+      'timestamp',
+    ],
     StencilOperation: [
       'keep',
       'zero',
@@ -487,6 +493,7 @@ var LibraryWebGPU = {
   {{{ gpu.makeReferenceRelease('Sampler') }}}
   {{{ gpu.makeReferenceRelease('Texture') }}}
   {{{ gpu.makeReferenceRelease('TextureView') }}}
+  {{{ gpu.makeReferenceRelease('QuerySet') }}}
 
   {{{ gpu.makeReferenceRelease('BindGroupLayout') }}}
   {{{ gpu.makeReferenceRelease('PipelineLayout') }}}
@@ -499,8 +506,9 @@ var LibraryWebGPU = {
 
   // *Destroy
 
-  wgpuBufferDestroy: function(bufferId) { WebGPU.mgrBuffer.get(bufferId).destroy(); },
-  wgpuTextureDestroy: function(textureId) { WebGPU.mgrTexture.get(textureId).destroy(); },
+  wgpuBufferDestroy: function(bufferId) { WebGPU.mgrBuffer.get(bufferId)["destroy"](); },
+  wgpuTextureDestroy: function(textureId) { WebGPU.mgrTexture.get(textureId)["destroy"](); },
+  wgpuQuerySetDestroy: function(querySetId) { WebGPU.mgrQuerySet.get(querySetId)["destroy"](); },
 
   // wgpuDevice
 
@@ -537,41 +545,26 @@ var LibraryWebGPU = {
 
   wgpuDeviceCreateBuffer: function(deviceId, descriptor) {
     {{{ gpu.makeCheckDescriptor('descriptor') }}}
+
+    var mappedAtCreation = {{{ gpu.makeGetBool('descriptor', C_STRUCTS.WGPUBufferDescriptor.mappedAtCreation) }}};
+
     var desc = {
       "label": undefined,
       "usage": {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUBufferDescriptor.usage) }}},
       "size": {{{ gpu.makeGetU64('descriptor', C_STRUCTS.WGPUBufferDescriptor.size) }}},
+      "mappedAtCreation": mappedAtCreation,
     };
     var labelPtr = {{{ makeGetValue('descriptor', C_STRUCTS.WGPUBufferDescriptor.label, '*') }}};
     if (labelPtr) desc["label"] = UTF8ToString(labelPtr);
 
     var device = WebGPU["mgrDevice"].get(deviceId);
-    return WebGPU.mgrBuffer.create(device["createBuffer"](desc));
-  },
-
-  wgpuDeviceCreateBufferMapped: function(returnPtr, deviceId, descriptor) {
-    {{{ gpu.makeCheckDescriptor('descriptor') }}}
-    var desc = {
-      "usage": {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUBufferDescriptor.usage) }}},
-      "size": {{{ gpu.makeGetU64('descriptor', C_STRUCTS.WGPUBufferDescriptor.size) }}},
-    };
-
-    var device = WebGPU["mgrDevice"].get(deviceId);
-    var bufferMapped = device["createBufferMapped"](desc);
-    var buffer = bufferMapped[0];
-    var mapped = bufferMapped[1];
-
-    var bufferWrapper = {};
-    var bufferId = WebGPU.mgrBuffer.create(buffer, bufferWrapper);
-    WebGPU.trackMapWrite(bufferWrapper, mapped);
-
-    var dataLength_high = (mapped.byteLength / 0x100000000) | 0;
-    var dataLength_low = mapped.byteLength | 0;
-
-    {{{ makeSetValue('returnPtr', C_STRUCTS.WGPUCreateBufferMappedResult.buffer, 'bufferId', '*') }}}
-    {{{ makeSetValue('returnPtr', C_STRUCTS.WGPUCreateBufferMappedResult.dataLength + 0, 'dataLength_low', 'i32') }}}
-    {{{ makeSetValue('returnPtr', C_STRUCTS.WGPUCreateBufferMappedResult.dataLength + 4, 'dataLength_high', 'i32') }}}
-    {{{ makeSetValue('returnPtr', C_STRUCTS.WGPUCreateBufferMappedResult.data, 'bufferWrapper.mapWriteSrc', '*') }}}
+    var id = WebGPU.mgrBuffer.create(device["createBuffer"](desc));
+    if (mappedAtCreation) {
+      var bufferWrapper = WebGPU.mgrBuffer.objects[id];
+      bufferWrapper.mapMode = 2 /* WGPUMapMode_Write */;
+      bufferWrapper.onUnmap = [];
+    }
+    return id;
   },
 
   wgpuDeviceCreateTexture: function(deviceId, descriptor) {
@@ -580,7 +573,6 @@ var LibraryWebGPU = {
     var desc = {
       "label": undefined,
       "size": WebGPU.makeExtent3D(descriptor + {{{ C_STRUCTS.WGPUTextureDescriptor.size }}}),
-      "arrayLayerCount": {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUTextureDescriptor.arrayLayerCount) }}},
       "mipLevelCount": {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUTextureDescriptor.mipLevelCount) }}},
       "sampleCount": {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUTextureDescriptor.sampleCount) }}},
       "dimension": WebGPU.TextureDimension[
@@ -648,6 +640,8 @@ var LibraryWebGPU = {
           {{{ gpu.makeGetBool('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.multisampled) }}},
         "hasDynamicOffset":
           {{{ gpu.makeGetBool('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.hasDynamicOffset) }}},
+        "minBufferBindingSize":
+          {{{ gpu.makeGetU64('entryPtr', C_STRUCTS.WGPUBindGroupLayoutEntry.minBufferBindingSize) }}},
       };
     }
 
@@ -764,6 +758,33 @@ var LibraryWebGPU = {
     return WebGPU.mgrPipelineLayout.create(device["createPipelineLayout"](desc));
   },
 
+  wgpuDeviceCreateQuerySet: function(deviceId, descriptor) {
+    {{{ gpu.makeCheckDescriptor('descriptor') }}}
+
+    var pipelineStatistics;
+    var pipelineStatisticsCount =
+      {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUQuerySetDescriptor.pipelineStatisticsCount) }}};
+    if (pipelineStatisticsCount) {
+      var pipelineStatisticsPtr =
+        {{{ makeGetValue('descriptor', C_STRUCTS.WGPUQuerySetDescriptor.pipelineStatistics, '*') }}};
+      pipelineStatistics = [];
+      for (var i = 0; i < pipelineStatisticsCount; ++i) {
+        pipelineStatistics.push(WebGPU.PipelineStatisticName[
+          {{{ gpu.makeGetU32('pipelineStatisticsPtr', '4 * i') }}}]);
+      }
+    }
+
+    var desc = {
+      "type": WebGPU.QueryType[
+        {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUQuerySetDescriptor.type) }}}],
+      "count": {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUQuerySetDescriptor.count) }}},
+      "pipelineStatistics": pipelineStatistics,
+    };
+
+    var device = WebGPU["mgrDevice"].get(deviceId);
+    device["createQuerySet"](descriptor);
+  },
+
   wgpuDeviceCreateComputePipeline: function(deviceId, descriptor) {
     {{{ gpu.makeCheckDescriptor('descriptor') }}}
 
@@ -963,7 +984,12 @@ var LibraryWebGPU = {
     if (sType === {{{ gpu.SType.ShaderModuleSPIRVDescriptor }}}) {
       var count = {{{ gpu.makeGetU32('nextInChainPtr', C_STRUCTS.WGPUShaderModuleSPIRVDescriptor.codeSize) }}};
       var start = {{{ makeGetValue('nextInChainPtr', C_STRUCTS.WGPUShaderModuleSPIRVDescriptor.code, '*') }}};
+#if USE_PTHREADS
+      // Chrome can't currently handle a SharedArrayBuffer view here, so make a copy.
+      desc["code"] = HEAPU32.slice(start >> 2, (start >> 2) + count);
+#else
       desc["code"] = HEAPU32.subarray(start >> 2, (start >> 2) + count);
+#endif
     } else if (sType === {{{ gpu.SType.ShaderModuleWGSLDescriptor }}}) {
       var sourcePtr = {{{ makeGetValue('nextInChainPtr', C_STRUCTS.WGPUShaderModuleWGSLDescriptor.source, '*') }}};
       if (sourcePtr) {
@@ -992,7 +1018,7 @@ var LibraryWebGPU = {
       if (ev.error instanceof GPUValidationError) type = Validation;
       else if (ev.error instanceof GPUOutOfMemoryError) type = OutOfMemory;
       var messagePtr = allocateUTF8(ev.error.message);
-      dynCall('viii', callback, [type, messagePtr, userdata]);
+      dynCall_viii(callback, type, messagePtr, userdata);
       _free(messagePtr);
     };
   },
@@ -1004,13 +1030,10 @@ var LibraryWebGPU = {
     var fence = WebGPU.mgrFence.get(fenceId);
     var completionValue = {{{ gpu.makeU64ToNumber('completionValue_low', 'completionValue_high') }}};
 
-    var WEBGPU_FENCE_COMPLETION_STATUS_SUCCESS = 0;
-    var WEBGPU_FENCE_COMPLETION_STATUS_ERROR = 1;
-
     fence.onCompletion(completionValue).then(function() {
-      dynCall('vii', callback, [WEBGPU_FENCE_COMPLETION_STATUS_SUCCESS, userdata]);
+      dynCall_vii(callback, 0 /* WEBGPU_FENCE_COMPLETION_STATUS_SUCCESS */, userdata);
     }, function() {
-      dynCall('vii', callback, [WEBGPU_FENCE_COMPLETION_STATUS_ERROR, userdata]);
+      dynCall_vii(callback, 1 /* WEBGPU_FENCE_COMPLETION_STATUS_ERROR */, userdata);
     });
   },
 
@@ -1037,7 +1060,7 @@ var LibraryWebGPU = {
     var queue = WebGPU.mgrQueue.get(queueId);
     var fence = WebGPU.mgrFence.get(fenceId);
     var signalValue = {{{ gpu.makeU64ToNumber('signalValue_low', 'signalValue_high') }}};
-    queue.signal(fence, signalValue);
+    queue["signal"](fence, signalValue);
   },
 
   wgpuQueueSubmit: function(queueId, commandCount, commands) {
@@ -1047,16 +1070,32 @@ var LibraryWebGPU = {
     var queue = WebGPU.mgrQueue.get(queueId);
     var cmds = Array.from(HEAP32.subarray(commands >> 2, (commands >> 2) + commandCount),
       function(id) { return WebGPU.mgrCommandBuffer.get(id); });
-    queue.submit(cmds);
+    queue["submit"](cmds);
+  },
+
+  wgpuQueueWriteBuffer: function(queueId,
+      bufferId, {{{ defineI64Param('bufferOffset') }}}, data, size) {
+    {{{ receiveI64ParamAsI32s('bufferOffset') }}}
+
+    var queue = WebGPU.mgrQueue.get(queueId);
+    var buffer = WebGPU.mgrBuffer.get(bufferId);
+    var bufferOffset = {{{ gpu.makeU64ToNumber('bufferOffset_low', 'bufferOffset_high') }}};
+    queue["writeBuffer"](buffer, bufferOffset, HEAPU8, data, size);
+  },
+
+  wgpuQueueWriteTexture: function(queueId,
+      destinationPtr, data, _dataSize, dataLayoutPtr, writeSizePtr) {
+    var queue = WebGPU.mgrQueue.get(queueId);
+
+    var destination = WebGPU.makeTextureCopyView(destinationPtr);
+    var dataLayout = WebGPU.makeTextureDataLayout(dataLayoutPtr);
+    dataLayout["offset"] += data;
+    var writeSize = WebGPU.makeExtent3D(writeSizePtr);
+    queue["writeTexture"](destination, HEAPU8, dataLayout, writeSize);
   },
 
   // wgpuCommandEncoder
 
-  wgpuCommandEncoderFinish: function(encoderId) {
-    var commandEncoder = WebGPU.mgrCommandEncoder.get(encoderId);
-    return WebGPU.mgrCommandBuffer.create(commandEncoder["finish"]());
-  },
-
   wgpuDeviceCreateRenderBundleEncoder: function(deviceId, descriptor) {
     {{{ gpu.makeCheck('descriptor') }}}
 
@@ -1151,9 +1190,11 @@ var LibraryWebGPU = {
         "depthStoreOp": WebGPU.StoreOp[
           {{{ gpu.makeGetU32('dsaPtr', C_STRUCTS.WGPURenderPassDepthStencilAttachmentDescriptor.depthStoreOp) }}}],
         "depthLoadValue": depthLoadValue,
+        "depthReadOnly": {{{ gpu.makeGetBool('dsaPtr', C_STRUCTS.WGPURenderPassDepthStencilAttachmentDescriptor.depthReadOnly) }}},
         "stencilStoreOp": WebGPU.StoreOp[
           {{{ gpu.makeGetU32('dsaPtr', C_STRUCTS.WGPURenderPassDepthStencilAttachmentDescriptor.stencilStoreOp) }}}],
         "stencilLoadValue": stencilLoadValue,
+        "stencilReadOnly": {{{ gpu.makeGetBool('dsaPtr', C_STRUCTS.WGPURenderPassDepthStencilAttachmentDescriptor.stencilReadOnly) }}},
       };
     }
 
@@ -1166,6 +1207,8 @@ var LibraryWebGPU = {
           {{{ makeGetValue('descriptor', C_STRUCTS.WGPURenderPassDescriptor.colorAttachments, '*') }}}),
         "depthStencilAttachment": makeDepthStencilAttachment(
           {{{ makeGetValue('descriptor', C_STRUCTS.WGPURenderPassDescriptor.depthStencilAttachment, '*') }}}),
+        "occlusionQuerySet": WebGPU.mgrQuerySet.get(
+          {{{ makeGetValue('descriptor', C_STRUCTS.WGPURenderPassDescriptor.occlusionQuerySet, '*') }}}),
       };
       var labelPtr = {{{ makeGetValue('descriptor', C_STRUCTS.WGPURenderPassDescriptor.label, '*') }}};
       if (labelPtr) desc["label"] = UTF8ToString(labelPtr);
@@ -1212,60 +1255,133 @@ var LibraryWebGPU = {
       WebGPU.makeTextureCopyView(srcPtr), WebGPU.makeTextureCopyView(dstPtr), copySize);
   },
 
+  wgpuCommandEncoderResolveQuerySet: function(encoderId, querySetId, firstQuery, queryCount,
+      destinationId, {{{ defineI64Param('destinationOffset') }}}) {
+    {{{ receiveI64ParamAsI32s('destinationOffset') }}}
+    var commandEncoder = WebGPU.mgrCommandEncoder.get(encoderId);
+    var querySet = WebGPU.mgrQuerySet.get(querySetId);
+    var destination = WebGPU.mgrBuffer.get(destinationId);
+    var destinationOffset = {{{ gpu.makeU64ToNumber('destinationOffset_low', 'destinationOffset_high') }}};
+
+    commandEncoder["resolveQuerySet"](querySet, firstQuery, queryCount, destination, destinationOffset);
+  },
+
+  wgpuCommandEncoderWriteTimestamp: function(encoderId, querySetId, queryIndex) {
+    var commandEncoder = WebGPU.mgrCommandEncoder.get(encoderId);
+    var querySet = WebGPU.mgrQuerySet.get(querySetId);
+    commandEncoder["writeTimestamp"](querySet, queryIndex);
+  },
+
+  wgpuCommandEncoderPushDebugGroup: function(encoderId, groupLabelPtr) {
+    var encoder = WebGPU.mgrCommandEncoder.get(encoderId);
+    encoder["pushDebugGroup"](UTF8ToString(groupLabelPtr));
+  },
+  wgpuCommandEncoderPopDebugGroup: function(encoderId) {
+    var encoder = WebGPU.mgrCommandEncoder.get(encoderId);
+    encoder["popDebugGroup"]();
+  },
+  wgpuCommandEncoderInsertDebugMarker: function(encoderId, markerLabelPtr) {
+    var encoder = WebGPU.mgrCommandEncoder.get(encoderId);
+    encoder["insertDebugMarker"](UTF8ToString(markerLabelPtr));
+  },
+
+  wgpuCommandEncoderFinish: function(encoderId) {
+    var commandEncoder = WebGPU.mgrCommandEncoder.get(encoderId);
+    return WebGPU.mgrCommandBuffer.create(commandEncoder["finish"]());
+  },
+
+  // wgpuComputePipeline
+
+  wgpuComputePipelineGetBindGroupLayout(pipelineId, groupIndex) {
+    var pipeline = WebGPU.mgrComputePipeline.get(pipelineId);
+    return WebGPU.mgrBindGroupLayout.create(pipeline["getBindGroupLayout"](groupIndex));
+  },
+
+  // wgpuRenderPipeline
+
+  wgpuRenderPipelineGetBindGroupLayout(pipelineId, groupIndex) {
+    var pipeline = WebGPU.mgrRenderPipeline.get(pipelineId);
+    return WebGPU.mgrBindGroupLayout.create(pipeline["getBindGroupLayout"](groupIndex));
+  },
+
   // wgpuBuffer
 
-  wgpuBufferSetSubData: function(bufferId, {{{ defineI64Param('start') }}}, {{{ defineI64Param('count') }}}, data) {
-    {{{ receiveI64ParamAsI32s('start') }}}
-    {{{ receiveI64ParamAsI32s('count') }}}
-    var buffer = WebGPU.mgrBuffer.get(bufferId);
-    var start = {{{ gpu.makeU64ToNumber('start_low', 'start_high') }}};
-    var count = {{{ gpu.makeU64ToNumber('count_low', 'count_high') }}};
-    buffer["setSubData"](start, HEAPU8, data, count);
-  },
-
-  wgpuBufferMapReadAsync: function(bufferId, callback, userdata) {
-    var bufferEntry = WebGPU.mgrBuffer.objects[bufferId];
-    bufferEntry.mapped = 'write';
-    var buffer = bufferEntry.object;
-
-    buffer["mapReadAsync"]().then(function(mapped) {
-      var WEBGPU_BUFFER_MAP_ASYNC_STATUS_SUCCESS = 0;
-      var data = _malloc(mapped.byteLength);
-      HEAPU8.set(new Uint8Array(mapped), data);
-      var dataLength_high = (mapped.byteLength / 0x100000000) | 0;
-      var dataLength_low = mapped.byteLength | 0;
-      // WGPUBufferMapAsyncStatus status, const void* data, uint64_t dataLength, void* userdata
-      dynCall('viiji', callback, [WEBGPU_BUFFER_MAP_ASYNC_STATUS_SUCCESS, data, {{{ sendI64Argument('dataLength_low', 'dataLength_high') }}}, userdata]);
-    }, function() {
-      // TODO(kainino0x): Figure out how to pick other error status values.
-      var WEBGPU_BUFFER_MAP_ASYNC_STATUS_ERROR = 1;
-      dynCall('viiji', callback, [WEBGPU_BUFFER_MAP_ASYNC_STATUS_ERROR, 0, 0, 0, userdata]);
-    });
-  },
-
-  wgpuBufferMapWriteAsync: function(bufferId, callback, userdata) {
+  wgpuBufferGetConstMappedRange: function(bufferId, offset, size) {
     var bufferWrapper = WebGPU.mgrBuffer.objects[bufferId];
+
+    var mapped;
+    try {
+      mapped = bufferWrapper.object["getMappedRange"](offset, size);
+    } catch (ex) {
+      // TODO(kainino0x): Somehow inject a validation error?
+      return 0;
+    }
+
+    var data = _malloc(mapped.byteLength);
+    HEAPU8.set(new Uint8Array(mapped), data);
+    bufferWrapper.onUnmap.push(function() {
+      _free(data);
+    });
+    return data;
+  },
+
+  wgpuBufferGetMappedRange: function(bufferId, offset, size) {
+    var bufferWrapper = WebGPU.mgrBuffer.objects[bufferId];
+
+    if (bufferWrapper.mapMode !== 2 /* WGPUMapMode_Write */) {
+#if ASSERTIONS
+      abort("GetMappedRange called, but buffer not mapped for writing");
+#endif
+      // TODO(kainino0x): Somehow inject a validation error?
+      return 0;
+    }
+
+    var mapped;
+    try {
+      mapped = bufferWrapper.object["getMappedRange"](offset, size);
+    } catch (ex) {
+      // TODO(kainino0x): Somehow inject a validation error?
+      return 0;
+    }
+
+    var data = _malloc(mapped.byteLength);
+    HEAPU8.fill(0, data, mapped.byteLength);
+    bufferWrapper.onUnmap.push(function() {
+      new Uint8Array(mapped).set(HEAPU8.subarray(data, data + mapped.byteLength));
+      _free(data);
+    });
+    return data;
+  },
+
+  wgpuBufferMapAsync: function(bufferId, mode, offset, size, callback, userdata) {
+    var bufferWrapper = WebGPU.mgrBuffer.objects[bufferId];
+    bufferWrapper.mapMode = mode;
+    bufferWrapper.onUnmap = [];
     var buffer = bufferWrapper.object;
 
-    var WEBGPU_BUFFER_MAP_ASYNC_STATUS_SUCCESS = 0;
-    var WEBGPU_BUFFER_MAP_ASYNC_STATUS_ERROR = 1;
-    buffer["mapWriteAsync"]().then(function(mapped) {
-      WebGPU.trackMapWrite(bufferWrapper, mapped);
+    // `callback` takes (WGPUBufferMapAsyncStatus status, void * userdata)
 
-      var data = bufferWrapper.mapWriteSrc;
-      var dataLength_high = (mapped.byteLength / 0x100000000) | 0;
-      var dataLength_low = mapped.byteLength | 0;
-      // WGPUBufferMapAsyncStatus status, void* data, uint64_t dataLength, void* userdata
-      dynCall('viiji', callback, [WEBGPU_BUFFER_MAP_ASYNC_STATUS_SUCCESS, data, {{{ sendI64Argument('dataLength_low', 'dataLength_high') }}}, userdata]);
+    buffer["mapAsync"](mode, offset, size).then(function() {
+      dynCall_vii(callback, 0 /* WGPUBufferMapAsyncStatus_Success */, userdata);
     }, function() {
       // TODO(kainino0x): Figure out how to pick other error status values.
-      dynCall('viiji', callback, [WEBGPU_BUFFER_MAP_ASYNC_STATUS_ERROR, 0, 0, 0, userdata]);
+      dynCall_vii(callback, 1 /* WGPUBufferMapAsyncStatus_Error */, userdata);
     });
   },
 
   wgpuBufferUnmap: function(bufferId) {
     var bufferWrapper = WebGPU.mgrBuffer.objects[bufferId];
-    WebGPU.trackUnmap(bufferWrapper);
+
+    if (!bufferWrapper.onUnmap) {
+      // Already unmapped
+      return;
+    }
+
+    for (var i = 0; i < bufferWrapper.onUnmap.length; ++i) {
+      bufferWrapper.onUnmap[i]();
+    }
+    bufferWrapper.onUnmap = undefined;
+
     bufferWrapper.object["unmap"]();
   },
 
@@ -1315,6 +1431,7 @@ var LibraryWebGPU = {
     var pipeline = WebGPU.mgrComputePipeline.get(pipelineId);
     pass["setPipeline"](pipeline);
   },
+
   wgpuComputePassEncoderDispatch: function(passId, x, y, z) {
     var pass = WebGPU.mgrComputePassEncoder.get(passId);
     pass["dispatch"](x, y, z);
@@ -1326,6 +1443,36 @@ var LibraryWebGPU = {
     var pass = WebGPU.mgrComputePassEncoder.get(passId);
     pass["dispatchIndirect"](indirectBuffer, indirectOffset);
   },
+
+  wgpuComputePassEncoderBeginPipelineStatisticsQuery: function(passId, querySetId, queryIndex) {
+    var pass = WebGPU.mgrComputePassEncoder.get(passId);
+    var querySet = WebGPU.mgrQuerySet.get(querySetId);
+    pass["beginPipelineStatisticsQuery"](querySet, queryIndex);
+  },
+  wgpuComputePassEncoderEndPipelineStatisticsQuery: function(passId) {
+    var pass = WebGPU.mgrComputePassEncoder.get(passId);
+    pass["endPipelineStatisticsQuery"]();
+  },
+
+  wgpuComputePassEncoderWriteTimestamp: function(encoderId, querySetId, queryIndex) {
+    var pass = WebGPU.mgrComputePassEncoder.get(passId);
+    var querySet = WebGPU.mgrQuerySet.get(querySetId);
+    pass["writeTimestamp"](querySet, queryIndex);
+  },
+
+  wgpuComputePassEncoderPushDebugGroup: function(encoderId, groupLabelPtr) {
+    var encoder = WebGPU.mgrComputePassEncoder.get(encoderId);
+    encoder["pushDebugGroup"](UTF8ToString(groupLabelPtr));
+  },
+  wgpuComputePassEncoderPopDebugGroup: function(encoderId) {
+    var encoder = WebGPU.mgrComputePassEncoder.get(encoderId);
+    encoder["popDebugGroup"]();
+  },
+  wgpuComputePassEncoderInsertDebugMarker: function(encoderId, markerLabelPtr) {
+    var encoder = WebGPU.mgrComputePassEncoder.get(encoderId);
+    encoder["insertDebugMarker"](UTF8ToString(markerLabelPtr));
+  },
+
   wgpuComputePassEncoderEndPass: function(passId) {
     var pass = WebGPU.mgrComputePassEncoder.get(passId);
     pass["endPass"]();
@@ -1377,6 +1524,7 @@ var LibraryWebGPU = {
     var pass = WebGPU.mgrRenderPassEncoder.get(passId);
     pass["setVertexBuffer"](slot, WebGPU.mgrBuffer.get(bufferId), offset, size);
   },
+
   wgpuRenderPassEncoderDraw: function(passId, vertexCount, instanceCount, firstVertex, firstInstance) {
     var pass = WebGPU.mgrRenderPassEncoder.get(passId);
     pass["draw"](vertexCount, instanceCount, firstVertex, firstInstance);
@@ -1399,6 +1547,7 @@ var LibraryWebGPU = {
     var pass = WebGPU.mgrRenderPassEncoder.get(passId);
     pass["drawIndexedIndirect"](indirectBuffer, indirectOffset);
   },
+
   wgpuRenderPassEncoderExecuteBundles: function(passId, count, bundlesPtr) {
     var pass = WebGPU.mgrRenderPassEncoder.get(passId);
 
@@ -1410,6 +1559,45 @@ var LibraryWebGPU = {
       function(id) { return WebGPU.mgrRenderBundle.get(id); });
     pass["executeBundles"](bundles);
   },
+
+  wgpuRenderPassEncoderBeginOcclusionQuery: function(passId, queryIndex) {
+    var pass = WebGPU.mgrRenderPassEncoder.get(passId);
+    pass["beginOcclusionQuery"](queryIndex);
+  },
+  wgpuRenderPassEncoderEndOcclusionQuery: function(passId) {
+    var pass = WebGPU.mgrRenderPassEncoder.get(passId);
+    pass["endOcclusionQuery"]();
+  },
+
+  wgpuRenderPassEncoderBeginPipelineStatisticsQuery: function(passId, querySetId, queryIndex) {
+    var pass = WebGPU.mgrRenderPassEncoder.get(passId);
+    var querySet = WebGPU.mgrQuerySet.get(querySetId);
+    pass["beginPipelineStatisticsQuery"](querySet, queryIndex);
+  },
+  wgpuRenderPassEncoderEndPipelineStatisticsQuery: function(passId) {
+    var pass = WebGPU.mgrRenderPassEncoder.get(passId);
+    pass["endPipelineStatisticsQuery"]();
+  },
+
+  wgpuRenderPassEncoderWriteTimestamp: function(encoderId, querySetId, queryIndex) {
+    var pass = WebGPU.mgrRenderPassEncoder.get(passId);
+    var querySet = WebGPU.mgrQuerySet.get(querySetId);
+    pass["writeTimestamp"](querySet, queryIndex);
+  },
+
+  wgpuRenderPassEncoderPushDebugGroup: function(encoderId, groupLabelPtr) {
+    var encoder = WebGPU.mgrRenderPassEncoder.get(encoderId);
+    encoder["pushDebugGroup"](UTF8ToString(groupLabelPtr));
+  },
+  wgpuRenderPassEncoderPopDebugGroup: function(encoderId) {
+    var encoder = WebGPU.mgrRenderPassEncoder.get(encoderId);
+    encoder["popDebugGroup"]();
+  },
+  wgpuRenderPassEncoderInsertDebugMarker: function(encoderId, markerLabelPtr) {
+    var encoder = WebGPU.mgrRenderPassEncoder.get(encoderId);
+    encoder["insertDebugMarker"](UTF8ToString(markerLabelPtr));
+  },
+
   wgpuRenderPassEncoderEndPass: function(passId) {
     var pass = WebGPU.mgrRenderPassEncoder.get(passId);
     pass["endPass"]();
@@ -1448,6 +1636,7 @@ var LibraryWebGPU = {
     var pass = WebGPU.mgrRenderBundleEncoder.get(bundleId);
     pass["setVertexBuffer"](slot, WebGPU.mgrBuffer.get(bufferId), offset);
   },
+
   wgpuRenderBundleEncoderDraw: function(bundleId, vertexCount, instanceCount, firstVertex, firstInstance) {
     var pass = WebGPU.mgrRenderBundleEncoder.get(bundleId);
     pass["draw"](vertexCount, instanceCount, firstVertex, firstInstance);
@@ -1456,6 +1645,20 @@ var LibraryWebGPU = {
     var pass = WebGPU.mgrRenderBundleEncoder.get(bundleId);
     pass["drawIndexed"](indexCount, instanceCount, firstIndex, baseVertex, firstInstance);
   },
+
+  wgpuRenderBundleEncoderPushDebugGroup: function(encoderId, groupLabelPtr) {
+    var encoder = WebGPU.mgrRenderBundleEncoder.get(encoderId);
+    encoder["pushDebugGroup"](UTF8ToString(groupLabelPtr));
+  },
+  wgpuRenderBundleEncoderPopDebugGroup: function(encoderId) {
+    var encoder = WebGPU.mgrRenderBundleEncoder.get(encoderId);
+    encoder["popDebugGroup"]();
+  },
+  wgpuRenderBundleEncoderInsertDebugMarker: function(encoderId, markerLabelPtr) {
+    var encoder = WebGPU.mgrRenderBundleEncoder.get(encoderId);
+    encoder["insertDebugMarker"](UTF8ToString(markerLabelPtr));
+  },
+
   wgpuRenderBundleEncoderFinish: function(bundleId, descriptor) {
     var desc;
     if (descriptor) {
@@ -1470,34 +1673,43 @@ var LibraryWebGPU = {
 
   // Instance
 
+  wgpuCreateInstance: function() {
+#if ASSERTIONS
+    // Once implemented, Instance should create and own the object management tables.
+    abort('unimplemented (TODO)');
+#endif
+  },
+
   wgpuInstanceReference: function() {
 #if ASSERTIONS
-    assert(false, 'No WGPUInstance object should exist.');
+    abort('No WGPUInstance object should exist (TODO).');
 #endif
   },
   wgpuInstanceRelease: function() {
 #if ASSERTIONS
-    assert(false, 'No WGPUInstance object should exist.');
+    abort('No WGPUInstance object should exist (TODO).');
 #endif
   },
 
+  wgpuInstanceCreateSurface__deps: ['$findCanvasEventTarget'],
   wgpuInstanceCreateSurface: function(instanceId, descriptor) {
     {{{ gpu.makeCheck('descriptor') }}}
     {{{ gpu.makeCheck('instanceId === 0, "WGPUInstance is ignored"') }}}
     var nextInChainPtr = {{{ makeGetValue('descriptor', C_STRUCTS.WGPUSurfaceDescriptor.nextInChain, '*') }}};
 #if ASSERTIONS
     assert(nextInChainPtr !== 0);
-    assert({{{ gpu.SType.SurfaceDescriptorFromHTMLCanvasId }}} ===
+    assert({{{ gpu.SType.SurfaceDescriptorFromCanvasHTMLSelector }}} ===
       {{{ gpu.makeGetU32('nextInChainPtr', C_STRUCTS.WGPUChainedStruct.sType) }}});
 #endif
-    var descriptorFromHTMLCanvasId = nextInChainPtr;
+    var descriptorFromCanvasHTMLSelector = nextInChainPtr;
 
-    {{{ gpu.makeCheckDescriptor('descriptorFromHTMLCanvasId') }}}
-    var idPtr = {{{ makeGetValue('descriptorFromHTMLCanvasId', C_STRUCTS.WGPUSurfaceDescriptorFromHTMLCanvasId.id, '*') }}};
-    {{{ gpu.makeCheck('idPtr') }}}
-    var id = UTF8ToString(idPtr);
-    var canvas = document.getElementById(id);
+    {{{ gpu.makeCheckDescriptor('descriptorFromCanvasHTMLSelector') }}}
+    var selectorPtr = {{{ makeGetValue('descriptorFromCanvasHTMLSelector', C_STRUCTS.WGPUSurfaceDescriptorFromCanvasHTMLSelector.selector, '*') }}};
+    {{{ gpu.makeCheck('selectorPtr') }}}
+    var canvas = findCanvasEventTarget(selectorPtr);
+#if ASSERTIONS
     assert(canvas instanceof HTMLCanvasElement);
+#endif
 
     var labelPtr = {{{ makeGetValue('descriptor', C_STRUCTS.WGPUSurfaceDescriptor.label, '*') }}};
     if (labelPtr) canvas.surfaceLabelWebGPU = UTF8ToString(labelPtr);
@@ -1505,7 +1717,31 @@ var LibraryWebGPU = {
     return WebGPU.mgrSurface.create(canvas);
   },
 
-  // wgpuDeviceCreateSwapChain + WGPUSwapChain
+  wgpuInstanceProcessEvents: function() {
+#if ASSERTIONS
+    abort('wgpuInstanceProcessEvents is unsupported (use requestAnimationFrame via html5.h instead)');
+#endif
+  },
+  wgpuInstanceRequestAdapter: function() {
+    abort('unimplemented (TODO)');
+  },
+
+  // WGPUAdapter
+
+  wgpuAdapterGetProperties: function() {
+    abort('unimplemented (TODO)');
+  },
+  wgpuAdapterRequestDevice: function() {
+    abort('unimplemented (TODO)');
+  },
+
+  // WGPUSurface
+
+  wgpuSurfaceGetPreferredFormat: function(surfaceId, adapterId, callback, userdata) {
+    abort('unimplemented (TODO)');
+  },
+
+  // WGPUSwapChain
 
   wgpuDeviceCreateSwapChain: function(deviceId, surfaceId, descriptor) {
     {{{ gpu.makeCheckDescriptor('descriptor') }}}
@@ -1516,8 +1752,10 @@ var LibraryWebGPU = {
     canvas.height = {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUSwapChainDescriptor.height) }}};
 
     var ctx = canvas.getContext('gpupresent');
+#if ASSERTIONS
     assert({{{ gpu.PresentMode.Fifo }}} ===
       {{{ gpu.makeGetU32('descriptor', C_STRUCTS.WGPUSwapChainDescriptor.presentMode) }}});
+#endif
 
     var desc = {
       "label": undefined,
@@ -1532,18 +1770,25 @@ var LibraryWebGPU = {
 
     return WebGPU.mgrSwapChain.create(swapChain);
   },
+
   wgpuSwapChainGetCurrentTextureView: function(swapChainId) {
     var swapChain = WebGPU.mgrSwapChain.get(swapChainId);
     return WebGPU.mgrTextureView.create(swapChain["getCurrentTexture"]()["createView"]());
   },
-
-  // Unsupported (won't be implemented)
-
-  wgpuDeviceTick: function() {
-    assert(false, 'wgpuDeviceTick is unsupported (use requestAnimationFrame via html5.h instead)');
-  },
   wgpuSwapChainPresent: function() {
-    assert(false, 'wgpuSwapChainPresent is unsupported (use requestAnimationFrame via html5.h instead)');
+    // TODO: This could probably be emulated with ASYNCIFY.
+#if ASSERTIONS
+    abort('wgpuSwapChainPresent is unsupported (use requestAnimationFrame via html5.h instead)');
+#endif
+  },
+
+  // wgpuGetProcAddress
+
+  wgpuGetProcAddress: function() {
+#if ASSERTIONS
+    abort('unimplemented (TODO: issue #11526)');
+#endif
+    return 0;
   },
 };
 
